<?php

namespace HS\Providers;

use HS\User;
use Adldap\Adldap;
use Laravel\Fortify\Fortify;
use Illuminate\Http\Request;
use InvalidArgumentException;
use HS\Auth\Ldap\LdapUserProvider;
use Illuminate\Support\Facades\Log;
use Illuminate\Contracts\View\View;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Auth;
use Illuminate\Http\RedirectResponse;
use App\Actions\Fortify\CreateNewUser;
use Illuminate\Support\Facades\Cookie;
use Illuminate\Contracts\View\Factory;
use Illuminate\Support\ServiceProvider;
use Illuminate\Cache\RateLimiting\Limit;
use App\Actions\Fortify\ResetUserPassword;
use App\Actions\Fortify\UpdateUserPassword;
use Illuminate\Support\Facades\RateLimiter;
use Laravel\Fortify\Contracts\LogoutResponse;
use App\Actions\Fortify\UpdateUserProfileInformation;
use Illuminate\Contracts\Container\BindingResolutionException;
use Symfony\Component\Routing\Exception\RouteNotFoundException;

class FortifyServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        // Set a custom redirect when logging out.
        $this->app->instance(LogoutResponse::class, new class implements LogoutResponse {
            public function toResponse($request)
            {
                if (hs_setting('cAUTHTYPE', 'internal') == 'saml') {
                    return redirect()->route('altlogin');
                }
                return redirect('/login');
            }
        });
    }

    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        // Fortify::createUsersUsing(CreateNewUser::class);
        // Fortify::updateUserProfileInformationUsing(UpdateUserProfileInformation::class);
        // Fortify::updateUserPasswordsUsing(UpdateUserPassword::class);
        // Fortify::resetUserPasswordsUsing(ResetUserPassword::class);

        if (hs_setting('cAUTHTYPE', 'internal') == 'ldap_ad') {
            Fortify::authenticateUsing(function ($request) {
                $ldap = new Adldap;
                try {
                    $ldap->addProvider(config()->get('ldap'));
                    $canProceed = $ldap->getDefaultProvider()
                        ->auth()
                        ->attempt($request->sUsername, $request->password);
                    if ($canProceed) {
                        $user = User::where('sUsername', $request->sUsername)->first();
                        if ($user) {
                            return $user;
                        }
                        return false;
                    }
                    return false;
                } catch (AdldapException | \Exception $e) {
                    Log::error($e);
                    return false;
                }
            });
        }

        if (hs_setting('cAUTHTYPE', 'internal') == 'blackbox') {
            Fortify::authenticateUsing(function ($request) {
                $canProceed = true;
                $customCodeFile = customCodePath('BlackBox.php');

                if (! file_exists($customCodeFile)) {
                    Log::error('BlackBox is enabled but file '.$customCodeFile.' was not found.');
                    $canProceed = false;
                }

                require_once $customCodeFile;

                if (! function_exists('BlackBox')) {
                    Log::error('BlackBox is enabled but function "BlackBox" was not found within file '.$customCodeFile.' was not found.');
                    $canProceed = false;
                }

                if ($canProceed and BlackBox($request->sUsername, $request->password)) {
                    return User::where('sUsername', $request->sUsername)->first();
                }

                return false;
            });
        }

        Fortify::loginView(function () {
            return $this->loginView();
        });

        Fortify::confirmPasswordView(function () {
            return view('auth.passwords.confirm');
        });

        Fortify::twoFactorChallengeView(function () {
            return view('auth.passwords.two-factor-challenge');
        });

        RateLimiter::for('login', function (Request $request) {
            $email = (string) $request->email;
            return Limit::perMinute(5)->by($email.$request->ip());
        });

        RateLimiter::for('two-factor', function (Request $request) {
            return Limit::perMinute(5)->by($request->session()->get('login.id'));
        });
    }

    /**
     * Build our custom login view
     */
    protected function loginView()
    {
        if (request()->input('mobileauth')) {
            Cookie::queue('mobile-transact', 'true', 60);
        }

        if (hs_setting('cAUTHTYPE', 'internal') == 'saml') {
            // Prevent redirect loops:
            // If user is logged into portal, redirect to the primary portal
            // If they're already auth'ed as an admin, they'd be redirected already
            if (auth('portal')->check()) {
                return redirect()->to(cHOST.'/index.php?pg=request.history');
            } else {
                // Else assume they want to login as an admin
                return redirect()->route('saml2_login', 'hs');
            }
        }

        return view('auth.login', array_merge(
            $this->getLatestNews(),
            ['mobileauth' => false],
        ));
    }

    /**
     * Get the latest HelpSpot News
     * @return array
     * @throws BindingResolutionException
     */
    protected function getLatestNews()
    {
        $article = getHelpSpotRssArticleLink();

        if (! $article) {
            return ['article_link' => '', 'article_title' => ''];
        }

        return [
            'article_link'=> $article['link'].'?m='.createBlogLinkMetaQueryString(),
            'article_title' => $article['title'],
        ];
    }
}
